from __future__ import annotations
import sys

import loguru
from loguru import logger

from logrich.logger_assets import print_tbl, format_extra_obj
from logrich.config import config

# https://flaviocopes.com/rgb-color-codes/
# https://loguru.readthedocs.io/en/stable/api/logger.html#message
# https://loguru.readthedocs.io/en/stable/api/logger.html#loguru._logger.Logger.complete
# https://docs.python.org/3/library/string.html#format-string-syntax
# https://loguru.readthedocs.io/en/stable/resources/recipes.html#adapting-colors-and-format-of-logged-messages-dynamically
# https://loguru.readthedocs.io/en/stable/api/logger.html#loguru._logger.Logger.level
# https://encycolorpedia.ru/30d5c8 - colors

"""
    TRACE	    5	logrich.trace()
    DEBUG	    10	logrich.debug()
    INFO	    20	logrich.info()
    SUCCESS	    25	logrich.success()
    WARNING	    30	logrich.warning()
    ERROR	    40	logrich.error()
    CRITICAL	50	logrich.critical()
"""


logger.remove()


def format_exception_record(record: loguru.Record) -> str:
    """форматирует записи исключений"""
    message = record["message"]
    line = record["line"]
    level = record["level"]
    file = record["file"]
    msg = print_tbl(level=level, message=message, file=file, line=line, style="error")
    record["extra"]["msg"] = msg
    return config.LOGURU_EXCEPTION_FORMAT_LONG


def format_regular_record(record: loguru.Record) -> str:
    """форматирует все записи кроме исключений"""
    message = record["message"]
    line = record["line"]
    level = record["level"]
    file = record["file"]
    extra = record.get("extra")
    LOGURU_GENERIC_FORMAT = "{extra[msg]}"
    if level:
        msg = print_tbl(
            level=level, message=message, file=file, line=line, style=level.name.lower()
        )
        if extra:
            obj = extra.get("o")
            record["extra"]["msg"] = msg
            if obj:
                record["extra"]["obj"] = format_extra_obj(obj)
                sys.stdout.write("\033[F")  # back to previous line
                # sys.stdout.write("\033[K")  # clear line
                return LOGURU_GENERIC_FORMAT + "\n{extra[obj]}"
    return LOGURU_GENERIC_FORMAT


# для всех записей кроме исключений
logger.add(
    sink=sys.stdout,
    level=config.LOG_LEVEL,
    format=format_regular_record,
    backtrace=False,
    catch=False,
    filter=lambda record: record["extra"].get("name") == "all",
)


# только для исключений
# сначала работает synk, затем форматтер
logger.add(
    sink=sys.stderr,
    filter=lambda record: record["extra"].get("name") == "err",
    format=format_exception_record,
    backtrace=False,
    catch=True,
)

errlog = logger.opt(colors=True, exception=True).bind(name="err")
log = logger.opt(lazy=False, colors=True).bind(name="all")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['logrich']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=5.2.0,<6.0.0',
 'loguru>=0.6.0,<0.7.0',
 'pydantic[dotenv]>=1.9.1,<2.0.0',
 'rich>=12.4.4,<13.0.0',
 'toml==0.10.2',
 'watchdog==2.0.3']

setup_kwargs = {
    'name': 'logrich',
    'version': '0.6.0',
    'description': 'loguru + rich = logrich',
    'long_description': '### Логгер\n\n#### Совместная работа [loguru](https://loguru.readthedocs.io) & [rich](https://rich.readthedocs.io).\n\n[Screenshot logger](https://disk.yandex.ru/i/JexFefETxnJavA)  \n[Screenshot logger2](https://disk.yandex.ru/i/ubvT0kZbfS-Guw)\n\n![Screenshot logger](wiki/logrich_screenshot.png?raw=True "Screenshot")\n----\n![Screenshot logger too](wiki/logrich_screenshot2.png?raw=True "Screenshot")\n\nУровень вывода исключений определяется в переменных окружения.\nЦвета, ширины и шаблоны вывода также могут быть определены в окружении.\n\nОбработчики записей логов можно определять дополнительно, например запись в файл или отправка в канал.\n\n#### Использование\n\nсмотри [тест](tests/test_1.py) \n\n#### Как развернуть:\n\n```shell\ngit clone \ncd logrich\npoetry shell\npoetry install\n# создаём окружение\ncp template.env .env\n```\n\n#### Запустить тест(ы):\n\n```shell\npytest\n# монитор тестов\nptw\n```\n',
    'author': 'Dmitry Mavlin',
    'author_email': 'mavlind@list.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
